package be.jeffcheasey88.peeratcode.parser.java;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.lang.reflect.Modifier;
import java.util.LinkedList;
import java.util.List;

import be.jeffcheasey88.peeratcode.parser.Parser;
import be.jeffcheasey88.peeratcode.parser.Token;
import be.jeffcheasey88.peeratcode.parser.Tokenizer;
import be.jeffcheasey88.peeratcode.parser.state.StateTree;

public class JavaParser  extends Parser<JavaFile> {
	
	public static long time;
	
	public static void main(String[] args) throws Exception{
		File file = new File("C:\\Users\\jeffc\\eclipse-workspace\\peer-at-code-backend\\src\\be\\jeffcheasey88\\peeratcode\\routes\\Result.java");
		BufferedReader reader = new BufferedReader(new FileReader(file));
		
		time = System.currentTimeMillis();
		
		Parser<JavaFile> parser = new JavaParser();
		JavaFile jFile = new JavaFile();
		parser.parse(reader, jFile);

		System.out.println((System.currentTimeMillis()-time)+"ms");
		
		parser.build(new BufferedWriter(new FileWriter(new File("/home/ParserV2.txt"))));
	}
	
	public JavaParser(){
		Tokenizer tokenizer = new Tokenizer();
		
		
		StateTree<JavaFile> importState = new StateTree<>();
		importState.then((validator) -> {
			if(validator.validate(
					(token) -> token.getValue().equals("import"),
					(bag, token) -> bag.set(new LinkedList<>()))){
				
				validator.validate(
						(token) -> token.getValue().equals("static"),
						(bag, token) -> bag.set("static", null));
				
				while(validator.validate(
						(token) -> !token.getValue().equals(";"),
						(bag, token) -> bag.<List<Token>>get().add(token))
					);
				
				return validator.validate((token) -> token.getValue().equals(";"));
			}
			return false;
		}).<JavaElement>end((javafile, bag) -> javafile.addImport(bag));

		
		StateTree<JavaFile> main = new StateTree<>();
		
		main.then((validator) -> {			
			if(validator.validate(
					(token) -> token.getValue().equals("package"),
					(bag, token) -> bag.set(new LinkedList<>()))){
				
				while(validator.validate(
						(token) -> !token.getValue().equals(";"),
						(bag, token) -> bag.<List<Token>>get().add(token))
					);
				
				return validator.validate((token) -> token.getValue().equals(";"));
			}
			return false;
		}).<JavaElement>end((javafile, bag) -> javafile.setPackage(bag))
			.thenNoChild(importState);
		
		
		System.out.println((System.currentTimeMillis()-time)+"ms");
		
		setTokenizer(tokenizer);
		setStateTree(main);
	}
	
	public static int getModifier(String modifier){
		switch(modifier){
			case "public": return Modifier.PUBLIC;
			case "private": return Modifier.PRIVATE;
			case "protected": return Modifier.PROTECTED;
			case "static": return Modifier.STATIC;
			case "final": return Modifier.FINAL;
			case "synchronized": return Modifier.SYNCHRONIZED;
			case "volatile": return Modifier.VOLATILE;
			case "transient": return Modifier.TRANSIENT;
			case "native": return Modifier.NATIVE;
			case "abstract": return Modifier.ABSTRACT;
			case "strictfp": return Modifier.STRICT;
			default: break;
		}
		return 0;
	}
	
}
